import vcr

VCR = vcr.VCR(
    serializer="yaml",
    record_mode="once",
    match_on=["uri", "method"],
    filter_headers=["authorization", "token"],
    filter_post_data_parameters=["client_id", "client_secret"],
)



import vcr

VCR = vcr.VCR(
    serializer="yaml",
    record_mode="once",
    match_on=["uri", "method"],
    filter_headers=["authorization", "token"],
    filter_post_data_parameters=["client_id", "client_secret"],
    decode_compressed_response=True,
)


# def _getvcr(request, env):
#     if env in os.environ:
#         params = {'record_mode': 'all'}
#         # params = {'record_mode': 'new_episodes'}
#     else:
#         params = {'record_mode': 'none'}
#     path = str(Path(request.fspath).parent / 'cassettes' / str(request.function.__name__))
#     return VCR(cassette_library_dir=path,
#                filter_headers=['authorization', 'token'],
#                filter_post_data_parameters=['client_id', 'client_secret'],
#                filter_query_parameters=['access_key'],
#                **params)



import os

from .utils import get_setting

AZURE_CLIENT_ID = os.environ.get("AZURE_CLIENT_ID", "")
AZURE_CLIENT_SECRET = os.environ.get("AZURE_CLIENT_SECRET", "")

GRAPH_CLIENT_ID = os.environ.get("GRAPH_CLIENT_ID", AZURE_CLIENT_ID)
GRAPH_CLIENT_SECRET = os.environ.get("GRAPH_CLIENT_SECRET", AZURE_CLIENT_SECRET)

AZURE_SSL = True
AZURE_URL_EXPIRATION_SECS = 10800
AZURE_ACCESS_POLICY_EXPIRY = 10800  # length of time before signature expires in seconds
AZURE_ACCESS_POLICY_PERMISSION = "r"
AZURE_URL = "https://login.microsoftonline.com"
AZURE_GRAPH_API_BASE_URL = "https://graph.microsoft.com"
AZURE_GRAPH_API_VERSION = os.environ.get("AZURE_GRAPH_API_VERSION", "v1.0")  # beta
AZURE_GRAPH_API_PAGE_SIZE = 300

AZURE_POLICY = get_setting(["SOCIAL_AUTH_POLICY", "TENANT_POLICY"])
AZURE_TENANT_NAME = get_setting(["SOCIAL_AUTH_TENANT_NAME", "TENANT_NAME"])
AZURE_RESET_POLICY = get_setting(
    ["SOCIAL_AUTH_PASSWORD_RESET_POLICY", "PASSWORD_RESET_POLICY"]
)

UNICEF_EMAIL = os.environ.get("UNICEF_EMAIL", "@unicef.org")



from django.conf import settings
from django.http import HttpResponseRedirect

from social_core.exceptions import AuthCanceled, AuthMissingParameter
from social_django.middleware import SocialAuthExceptionMiddleware

from unicef_security.backends import UNICEFAzureADB2COAuth2

from . import config


class UNICEFSocialAuthExceptionMiddleware(SocialAuthExceptionMiddleware):
    """Middleware to ignore Forgot Password Exceptions"""

    def process_exception(self, request, exception):
        if isinstance(exception, (AuthCanceled, AuthMissingParameter)):
            return HttpResponseRedirect(self.get_redirect_uri(request, exception))
        else:
            raise exception

    def get_redirect_uri(self, request, exception):
        strategy = getattr(request, "social_strategy", None)
        error = request.GET.get("error", None)

        # This is what we should expect:
        # ['AADB2C90118: The user has forgotten their password.\r\n
        # Correlation ID: 7e8c3cf9-2fa7-47c7-8924-a1ea91137ba9\r\n
        # Timestamp: 2018-11-13 11:37:56Z\r\n']
        error_description = request.GET.get("error_description", None)
        if error == "access_denied" and error_description is not None:
            if "AADB2C90118" in error_description:
                auth_class = UNICEFAzureADB2COAuth2()
                redirect_home = auth_class.get_redirect_uri()
                reset_policy = config.AZURE_RESET_POLICY
                redirect_url = "".join(
                    [
                        auth_class.base_url,
                        "/oauth2/v2.0/",
                        f"authorize?p={reset_policy}",
                        f"&client_id={strategy.setting('KEY')}",
                        f"&nonce=defaultNonce&redirect_uri={redirect_home}",
                        "&scope=openid+email&response_type=code",
                    ]
                )
                return redirect_url

        # TODO: In case of password reset the state can't be verified figure out a way to log the user in after reset
        if error is None:
            return settings.LOGIN_URL

        strategy = getattr(request, "social_strategy", None)
        redirect_url = strategy.setting("LOGIN_ERROR_URL") + "?msgc=loginerror"
        return redirect_url



# Generated by Django 3.1.2 on 2020-10-28 01:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("unicef_security", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="first name"
            ),
        ),
    ]



from django.conf import settings


def get_setting(settings_list, default=None):
    for setting_name in settings_list:
        if value := getattr(settings, setting_name, None):
            return value
    return default



from django.contrib.auth.models import AbstractUser

from unicef_security.models import SecurityMixin


class User(SecurityMixin, AbstractUser):
    class Meta(AbstractUser.Meta):
        app_label = "demo"



import logging

from django import forms
from django.contrib import messages
from django.contrib.admin import SimpleListFilter
from django.contrib.auth.admin import UserAdmin
from django.template.response import TemplateResponse
from django.utils.translation import gettext_lazy as _

from admin_extra_buttons.decorators import button
from admin_extra_buttons.mixins import ExtraButtonsMixin

from .config import UNICEF_EMAIL
from .graph import default_group, Synchronizer, SyncResult

logger = logging.getLogger(__name__)


def is_superuser(request, *args, **kwargs):
    return request.user.is_superuser


class LoadUsersForm(forms.Form):
    emails = forms.CharField(widget=forms.Textarea)


class UNICEFUserFilter(SimpleListFilter):
    title = "UNICEF user filter"
    parameter_name = "email"

    def lookups(self, request, model_admin):
        return [
            ("unicef", "UNICEF"),
            ("external", "External"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "unicef":
            return queryset.filter(email__endswith=UNICEF_EMAIL)
        elif self.value() == "external":
            return queryset.exclude(email__endswith=UNICEF_EMAIL)
        return queryset


class UserAdminPlus(ExtraButtonsMixin, UserAdmin):
    list_display = [
        "username",
        "display_name",
        "email",
        "is_staff",
        "is_active",
        "is_superuser",
        "is_linked",
        "last_login",
    ]
    list_filter = ["is_superuser", "is_staff", "is_active", UNICEFUserFilter]
    search_fields = ["username", "display_name"]
    fieldsets = (
        (None, {"fields": (("username", "azure_id"), "password")}),
        (
            _("Personal info"),
            {
                "fields": (
                    (
                        "first_name",
                        "last_name",
                    ),
                    ("email", "display_name"),
                    ("job_title",),
                )
            },
        ),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                )
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "password1", "password2"),
            },
        ),
    )
    readonly_fields = ("azure_id", "job_title", "display_name")

    def is_linked(self, obj):
        return bool(obj.azure_id)

    is_linked.boolean = True

    @button(label="Sync")
    def sync_user(self, request, pk):
        obj = self.get_object(request, pk)
        try:
            synchronizer = Synchronizer()
            synchronizer.sync_user(obj)
        except BaseException as e:
            self.message_user(request, str(e), messages.ERROR)

        self.message_user(request, "User synchronized")

    @button(label="Link user", permissions=is_superuser)
    def link_user_data(self, request, pk):
        opts = self.model._meta
        ctx = {
            "opts": opts,
            "app_label": "security",
            "change": True,
            "is_popup": False,
            "save_as": False,
            "has_delete_permission": False,
            "has_add_permission": False,
            "has_change_permission": True,
        }
        obj = self.get_object(request, pk)
        synchronizer = Synchronizer()
        try:
            if request.method == "POST":
                if request.POST.get("selection"):
                    data = synchronizer.get_user(request.POST.get("selection"))
                    synchronizer.sync_user(obj, data["id"])
                    self.message_user(request, "User linked")
                    return None
                else:
                    ctx["message"] = "Select one entry to link"

            data = synchronizer.search_users(obj)
            ctx["data"] = data
            return TemplateResponse(request, "admin/link_user.html", ctx)

        except BaseException as e:
            self.message_user(request, str(e), messages.ERROR)

    @button()
    def load(self, request):
        opts = self.model._meta
        ctx = {
            "opts": opts,
            "app_label": "security",
            "change": True,
            "is_popup": False,
            "save_as": False,
            "has_delete_permission": False,
            "has_add_permission": False,
            "has_change_permission": True,
        }
        if request.method == "POST":
            form = LoadUsersForm(request.POST)
            if form.is_valid():
                synchronizer = Synchronizer()
                emails = form.cleaned_data["emails"].split()
                total_results = SyncResult()
                for email in emails:
                    result = synchronizer.fetch_users(
                        "startswith(mail,'%s')" % email, callback=default_group
                    )
                    total_results += result
                self.message_user(
                    request,
                    f"{len(total_results.created)} users have been created, "
                    f"{len(total_results.updated)} updated. "
                    f"{len(total_results.skipped)} invalid entries found.",
                )
        else:
            form = LoadUsersForm()
        ctx["form"] = form
        return TemplateResponse(request, "admin/load_users.html", ctx)

    @button(permissions=is_superuser)
    def ad(self, request, pk):
        obj = self.get_object(request, pk)
        try:
            synchronizer = Synchronizer()
            context = synchronizer.get_user(obj.username)
        except BaseException as e:
            self.message_user(request, str(e), messages.ERROR)

        return TemplateResponse(
            request, "admin/ad.html", {"ctx": context, "opts": self.model._meta}
        )



import os
from pathlib import Path

import pytest

from tests.fixtures import VCR
from unicef_security.graph import Synchronizer


@VCR.use_cassette(str(Path(__file__).parent / "vcr_cassettes/test_token.yml"))
@pytest.mark.skipif(os.environ.get("CIRCLECI") == "true", reason="Skip in CirlceCI")
def test_token():
    s = Synchronizer()
    assert s.access_token


@pytest.mark.xfail
@VCR.use_cassette(str(Path(__file__).parent / "vcr_cassettes/test_user_data.yml"))
def test_user_data():
    s = Synchronizer()
    info = s.get_user("sapostolico@unicef.org")
    assert info["displayName"]
    assert info["givenName"]
    assert info["id"]
    assert sorted(info.keys()) == sorted(
        [
            "@odata.context",
            "id",
            "businessPhones",
            "displayName",
            "givenName",
            "jobTitle",
            "mail",
            "mobilePhone",
            "officeLocation",
            "preferredLanguage",
            "surname",
            "userPrincipalName",
        ]
    )



from django.conf import settings
from django.template import Library
from django.urls import reverse

register = Library()


@register.simple_tag()
def ad_backend():
    return reverse('social:begin', args=[getattr(settings, "SOCIAL_AUTH_BACKEND_NAME", "azuread-tenant-oauth2")])



from social_core.tests.backends.test_azuread_b2c import AzureADB2COAuth2Test


class UNICEFAzureADTenantOAuth2ExtTest(AzureADB2COAuth2Test):
    backend_path = "unicef_security.backends.UNICEFAzureADTenantOAuth2Ext"



from django.conf import settings
from django.views.generic import RedirectView, TemplateView

from unicef_security import config
from unicef_security.config import UNICEF_EMAIL


class UnauthorizedView(TemplateView):
    template_name = "unauthorized.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context[
            "unicef_user"
        ] = self.request.user.is_authenticated and self.request.user.email.endswith(
            UNICEF_EMAIL
        )
        return context


class UNICEFLogoutView(RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        base_url = config.AZURE_URL
        tenant = config.AZURE_TENANT_NAME
        redirect_url = (
            f"{self.request.scheme}://{self.request.get_host()}{settings.LOGOUT_URL}"
        )

        return f"{base_url}/{tenant}/oauth2/v2.0/logout?post_logout_redirect_uri={redirect_url}"



import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "&1_8z#^^_nz1o4xuvz4wqq&m&gpe5uq_(=!%sk=cg5e&o^ljqu"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    "demo",
    "unicef_security",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "admin_extra_buttons",
    "constance",
    "constance.backends.database",
    "social_django",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

AUTHENTICATION_BACKENDS = (
    "unicef_security.backends.UNICEFAzureADB2COAuth2",
    "django.contrib.auth.backends.ModelBackend",
)

ROOT_URLCONF = "demo.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "demo.wsgi.application"


# Database
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}


# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True

HOST = "http://localhost"
# Static files (CSS, JavaScript, Images)
STATIC_URL = "/static/"

MEDIA_ROOT = "/tmp/"
VISION_LOGGER_MODEL = "vision.VisionLog"

AUTH_USER_MODEL = "demo.User"

CONSTANCE_CONFIG = {
    "DEFAULT_GROUP": ("", "test_grp"),
}

CONSTANCE_BACKEND = "constance.backends.database.DatabaseBackend"



from django.urls import reverse


def test_changelist(django_app, admin_user):
    url = reverse("admin:demo_user_changelist")
    res = django_app.get(url, user=admin_user)
    assert res.status_code == 200



from social_django.utils import load_backend, load_strategy

import pytest

from unicef_security.pipeline import create_unicef_user, get_username, social_details


@pytest.fixture()
def strategy():
    return load_strategy()


@pytest.fixture()
def backend(strategy):
    return load_backend(
        strategy=strategy, name="unicef-azuread-b2c-oauth2", redirect_uri="/"
    )


@pytest.fixture()
def response(auth_user):
    return {
        "id_token": "token",
        "token_type": "Bearer",
        "not_before": 1607541825,
        "id_token_expires_in": 3600,
        "profile_info": "profile_info",
        "scope": "openid",
        "access_token": "access_token",
        "exp": 1607545425,
        "nbf": 1607541825,
        "ver": "1.0",
        "iss": "https://tenant.b2clogin.com/1234567890/v2.0/",
        "sub": "abcdefgh",
        "aud": "abcdefgh",
        "acr": "b2c_1a_unicef_social_signup_signin",
        "iat": 1607541825,
        "auth_time": 1607541824,
        "given_name": "Given",
        "family_name": "Family",
        "name": "Given Family",
        "idp": "UNICEF Azure AD",
        "email": auth_user.email,
    }


@pytest.fixture()
def details(auth_user):
    return {
        "username": "Given Family",
        "email": auth_user.email,
        "fullname": "Given Family",
        "first_name": "Given",
        "last_name": "Family",
        "idp": "UNICEF Azure AD",
    }


@pytest.mark.django_db
def test_social_details(auth_user, backend, details, response):
    return_dict = social_details(backend, details, response)
    assert "idp" in return_dict["details"]
    assert return_dict["details"]["email"] == auth_user.email


@pytest.mark.django_db
def test_get_username(strategy, backend, details, auth_user):
    result_dict = get_username(strategy, details, backend, user=auth_user)
    assert "username" in result_dict
    assert result_dict["username"] == auth_user.email


@pytest.mark.django_db
def test_create_unicef_user_ok(strategy, details, backend):
    result_dict = create_unicef_user(strategy, details, backend)
    assert result_dict["is_new"]
    assert result_dict["user"]


@pytest.mark.django_db
def test_create_unicef_user_ko(strategy, details, backend):
    details["email"] = "test_undeliverable@example.com"
    result_dict = create_unicef_user(strategy, details, backend)
    assert not result_dict


@pytest.mark.django_db
def test_create_unicef_user_existing(strategy, details, backend):
    result_dict = create_unicef_user(strategy, details, backend, 1)
    assert "is_new" in result_dict
    assert not result_dict["is_new"]



import factory

from demo.models import User


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ("username",)

    username = factory.Sequence(lambda n: "m%03d@unicef.org" % n)

    last_name = factory.Faker("last_name")
    first_name = factory.Faker("first_name")

    email = factory.Sequence(lambda n: "m%03d@unicef.org" % n)
    password = "password"
    is_superuser = False
    is_active = True



#!/usr/bin/env python
import ast
import os
import re

from setuptools import find_packages, setup

HERE = os.path.abspath(os.path.dirname(__file__))
init = os.path.join(HERE, "src", "unicef_security", "__init__.py")

_version_re = re.compile(r"__version__\s+=\s+(.*)")
_name_re = re.compile(r"NAME\s+=\s+(.*)")

with open(init, "rb") as f:
    content = f.read().decode("utf-8")
    VERSION = str(ast.literal_eval(_version_re.search(content).group(1)))
    NAME = str(ast.literal_eval(_name_re.search(content).group(1)))


setup(
    name=NAME,
    version=VERSION,
    url="https://github.com/unicef/unicef-security",
    author="UNICEF",
    author_email="rapidpro@unicef.org",
    description="Provides Basic UNICEF User model and integration with Azure",
    platforms=["any"],
    license="Apache 2 License",
    classifiers=[
        "Environment :: Web Environment",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.11",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
    ],
    install_requires=[
        "celery",
        "cryptography",
        "django",
        "django-admin-extra-buttons",
        "django-constance",
        "django-countries",
        "django-picklefield",
        "requests",
        "social-auth-app-django",
        "PyJWT",
    ],
    extras_require={
        "test": [
            "django-webtest",
            "factory-boy",
            "flake8",
            "httpretty",
            "isort",
            "mock",
            "pytest",
            "pytest-cov",
            "pytest-django",
            "pytest-echo",
            "pytest-redis",
            "requests-mock",
            "unittest2",
            "vcrpy",
        ],
    },
    package_dir={"": "src"},
    packages=find_packages("src"),
    include_package_data=True,
)



from django.contrib import admin
from django.urls import include, re_path

urlpatterns = [
    re_path(r"^admin/", admin.site.urls),
    re_path(r"social/", include("social_django.urls", namespace="social")),
]



from demo.models import User


def test_user():
    user = User(username="user_name")
    assert str(user) == "user_name"
    assert user.label == "user_name"

    user = User(display_name="display_name")
    assert user.label == "display_name"

    user = User(first_name="first_name")
    assert user.label == "first_name"

    user = User(first_name="first_name", last_name="last_name")
    assert user.label == "first_name last_name:"
